#pragma once
#include <oxen_economy.h>

#include <ethyl/logs.hpp>
#include <ethyl/provider.hpp>
#include <string>
#include <unordered_set>
#include <variant>

#include "contracts.h"
#include "events.h"
#include "crypto/crypto.h"
#include "crypto/eth.h"
#include "cryptonote_config.h"

namespace eth {

event::StateChangeVariant getLogEvent(const ethyl::LogEntry& log);
inline bool is_state_change(const event::StateChangeVariant& v) {
    return v.index() > 0;
}

struct ContractServiceNode {
    bool good;
    uint64_t next;
    uint64_t prev;
    eth::address operatorAddr;
    bls_public_key pubkey;
    uint64_t leaveRequestTimestamp;
    uint64_t deposit;
    std::array<event::Contributor, oxen::MAX_CONTRIBUTORS_HF19> contributors;
    size_t contributorsSize;
};

class RewardsContract {
  public:
    // Constructor
    RewardsContract(cryptonote::network_type nettype, ethyl::Provider& provider);

    std::string_view address() { return contractAddress; }

    ContractServiceNode serviceNodes(
            uint64_t index, std::optional<uint64_t> blockNumber = std::nullopt);
    std::vector<uint64_t> getNonSigners(const std::unordered_set<bls_public_key>& bls_public_keys);
    std::vector<bls_public_key> getAllBLSPubkeys(uint64_t blockNumber);

  private:
    std::string contractAddress;
    ethyl::Provider& provider;
};

}  // namespace eth
